#ifndef _INCLUDES_H_
#define _INCLUDES_H_

// standardne hlavickove subory pre vsetky moduly **************************

// systemove kniznice ******************************************************
#include <stdlib.h>
#include <math.h>
#include <new>

// definicie pre Windows a Linux *******************************************
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#define PATHDIV '\\'
#else
#define LINUX
#define PATHDIV '/'
#endif

// SDL kniznica ************************************************************
// /usr/lib/libSDL.so
// /usr/lib/libSDL_image.so
#include <SDL.h>
#include <SDL_image.h>

// OpenGL, GLU a GLEXT kniznice ********************************************
// vyber konkretnych kniznic je zavisly na pouzitom grafickom hardveri
// a implementacii OpenGL (napr. pomocou Mesa kniznice)
// /usr/lib/libGL.so, alebo v zavislosti na GPU
// napr. /usr/lib/nvidia/libGL.so
// /usr/lib/libGLU.so
// v pripade pouzitia GLEXT kniznice sa pouzije subor suvisiaci s GPU
// napr. /usr/lib/xorg/modules/extensions/nvidia/libglx.so
#define GL_GLEXT_LEGACY
// informacia pre gl.h aby nepripojil obsah suboru glext.h
#define __glext_h__
// pripoji obsah suborov gl.h a glu.h, nepripoji obsah suboru SDL_opengl.h
#define NO_SDL_GLEXT
#include <SDL_opengl.h>
#undef __glext_h__
#include <GL/glext.h>

// rozne pomocne nastroje **************************************************
#include "tools.h"
// rozne geometricke a matematicke funkcie *********************************
#include "math3d.h"
// praca s texturami *******************************************************
#include "texture.h"

// trojuholnik *************************************************************
struct _tri {
 _tri() { }  // konstruktor
 ~_tri() { } // destruktor

 GLuint id; // identifikator trojuholnika

 vec4 c; // farba

 vec3 n; // normala
 GLfloat d; // prvok D rovnice roviny, ktoru tvori trojuholnik,
            // prvkami A, B, C su prvky normaly trojuholnika

 vec2 t[3];     // texturovacie suradnice
 GLuint tex1id; // identifikator 1. OpenGL textury
 GLuint tex2id; // identifikator 2. OpenGL textury

 vec2 l[3];   // texturovacie suradnice pre lightmapu
 GLuint lmid; // identifikator lightmapy

 vec3 v[3];  // suradnice vrcholov
 vec3 vn[3]; // normaly vo vrcholoch

 vec4 cn[3]; // prvky A, B, C "koliznych normal" a prvok D,
             // ktore spolu definuju rovnice rovin pre jednotlive
             // strany trojuholnika - vyuzivaju sa pre urcenie, ci
             // dany 3D bod patri/nepatri trojuholniku

 // minimalne a maximalne suradnice vrcholov na danych osiach
 // okolo trojuholnika sa tak vytvori pomyselny hranol
 vec2 minmaxx,minmaxy,minmaxz;

 // indikator spracovania, ak done = true trojuholnik
 // bol spracovany a nebude sa dalej spracovavat
 bool done;
};

// svetelny zdroj **********************************************************
struct _light {
 _light() { }  // konstruktor
 ~_light() { } // destruktor

 GLuint id; // identifikator svetelneho zdroja

 vec4 c;    // farba

 vec3 p;    // poloha
};

// globalne premenne *******************************************************

// definovane v engine.cpp:
extern int scr_w,scr_w2;
extern int scr_h,scr_h2;
extern int fsaa;

// definovane v interface.cpp:
extern CTexture texture;
extern GLuint tid[MAX_TEX_NUM];
extern GLuint lmid[MAX_LM_NUM];

// definovane v backend.cpp:
extern vector<_tri> tris;
extern vector<_light> lights;

#endif
