// hlavickove subory *******************************************************
#include "includes.h"
// vlastny .h subor
#include "text.h"

// konstruktor *************************************************************
CText::CText()
{
 // pociatocne nastavenie identifikatora textury a displaylistov
 texture_id=0; list_id=0;
 // obsah textury (farby pixelov)
 data=NULL; ptr=NULL;
}

// destruktor **************************************************************
CText::~CText()
{
 if (texture_id) glDeleteTextures(1,&texture_id); // zrusenie objektu textury
 if (list_id) glDeleteLists(list_id,256);         // zrusenie displaylistov
}

// inicializacia textu *****************************************************
int CText::init(const char *FONT_FILE)
{
 // SDL nacitanie obsahu suboru s texturou
 SDL_Surface *TextureImage;
 TextureImage=IMG_Load(FONT_FILE);
 if (TextureImage==NULL) {
  fprintf(stderr,"Nepodarilo sa nacitat texturu: %s\n",FONT_FILE);
  return(1);
 }

 // sirka a vyska textury
 w=(GLuint) TextureImage->w; h=(GLuint) TextureImage->h;
 if (w==0||h==0) return(1);

 // premenna data bude odkazovat na farbu pixelov textury
 data=(GLubyte *) TextureImage->pixels;
 if (data==NULL) return(1);

 // OpenGL vygenerovanie objektu textury
 glGenTextures(1,&texture_id);
 glBindTexture(GL_TEXTURE_2D,texture_id);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,w,h,0,GL_RGBA,GL_UNSIGNED_BYTE,data);

 // INFO: identifikator, sirka, vyska a nazov suboru textury
 fprintf(stderr,"Font textura: %u %u %u %s\n",texture_id,w,h,FONT_FILE);

 // rozmer jedneho znaku v texture
 step=w/16; // 512/16=32 = rozmer jedneho znaku v texture

 // vypocet pozicii jednotlivych znakov v ramci textury
 for (y=0,i=0;y<16;y++) {  // pre 16 pozicii v smere y
  for (x=0;x<16;x++,i++) { // pre 16 pozicii v smere x
   // najdenie lavej hranice, na ktorej zacina znak
   pos[i][0]=0;
   for (j=0;j<step;j++) {
    ptr=data+(x*step+y*w*step+j)*4;
    for (k=0;k<step;k++) { // kontrola vsetkych bodov v smere y
     if (*(ptr+3)!=0) break; // ak je najdena alfa zlozka farby
     ptr+=w*4;
    }
    if (k!=step) break; // ak bola najdena alfa zlozka farby
    pos[i][0]++; // postupuje sa zlava - doprava, hlada sa lava hranica znaku
   }
   pos[i][1]=0;
   // ak sa na danej pozicii nenasiel znak, pokracuje sa dalsou poziciou
   if (pos[i][0]==step) continue;
   // najdenie pravej hranice, na ktorej konci znak
   for (j=step-1;j>=0;j--) {
    ptr=data+(x*step+y*w*step+j)*4;
    for (k=0;k<step;k++) { // kontrola vsetkych bodov v smere y
     if (*(ptr+3)!=0) break; // ak je najdena alfa zlozka farby
     ptr+=w*4;
    }
    if (k!=step) break; // ak bola najdena alfa zlozka farby
    pos[i][1]++; // postupuje sa zprava - dolava, hlada sa prava hranica znaku
   }
   // vypocet sirky znaku
   pos[i][1]=step-pos[i][0]-pos[i][1];
  }
 }

 // OpenGL vygenerovanie 256-tich displaylistov
 list_id=glGenLists(256);

 // vytvorenie jednotlivych displaylistov
 for (y=0,i=0;y<16;y++) {  // pre 16 pozicii v smere y
  for (x=0;x<16;x++,i++) { // pre 16 pozicii v smere x
   // x suradnica textury
   s=(GLfloat) x/16.0f+(GLfloat) pos[i][0]/(GLfloat) w;
   // y suradnica textury
   t=(GLfloat) y/16.0f;
   // sirka znaku
   ds=(GLfloat) pos[i][1]/(GLfloat) w;
   // vyska znaku = cela vyska jednej pozicie
   dt=1.0f/16.0f;
   // ulozenie znaku do displaylistu list_id+i
   glNewList(list_id+i,GL_COMPILE);
    glBegin(GL_QUADS); // poradie vrcholov je CW pretoze pre vykreslenie bude pouzite glOrtho()
     glTexCoord2f(s,t);       glVertex2i(0,0);
     glTexCoord2f(s,t+dt);    glVertex2i(0,step);
     glTexCoord2f(s+ds,t+dt); glVertex2i(pos[i][1],step);
     glTexCoord2f(s+ds,t);    glVertex2i(pos[i][1],0);
    glEnd();
    // nastavenie polohy nasledujuceho znaku
    glTranslatef((GLfloat) pos[i][1],0.0f,0.0f);
   glEndList();
  }
 }

 // uvolnenie pouzitej pamate
 if (TextureImage!=NULL) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// vypis textu *************************************************************
void CText::draw(GLfloat X,GLfloat Y,const char *STRING)
{
 // priprava na vypis textu je vykonana na inom mieste! (v interface.cpp)
 // vykreslenie textu na poziciu x, y
 glTranslatef(X,Y,0.0f);
 // vykreslenie displaylistov prisluchajucich jednotlivym znakom
 for (l=0;*STRING!='\0';STRING++) {
  // z dovodu urychlenia sa nebude testovat na \n \r \t
  //if (*STRING=='\n') { // new line
  // l++;
  // glLoadIdentity();
  // glTranslatef(X,Y+(GLfloat) (step*l),0.0f);
  //} else if (*STRING=='\r') { // return
  //        glLoadIdentity();
  //        glTranslatef(X,Y+(GLfloat) (step*l),0.0f);
  //       } else if (*STRING=='\t') { // tabulator
  //               glTranslatef((GLfloat) step*2.0f,0.0f,0.0f);
  //              } else if (*STRING==' ') { // space
                       if (*STRING==' ') { // space
                        glTranslatef((GLfloat) step/2.0f,0.0f,0.0f);
                       } else { // znak
                          glCallList(list_id+*STRING);
                         }
 }
}

// vypis formatovaneho textu ***********************************************
void CText::drawf(GLfloat X,GLfloat Y,const char *FORMAT,...)
{
 // presun argumentov do pomocneho zasobnika na zaklade zvoleneho formatu vypis textu
 va_start(args,FORMAT);
 vsprintf(buf,FORMAT,args);
 va_end(args);
 // vypis textu
 draw(X,Y,buf);
}
